@@ -1,4 +1,8 @@
 from view import View, registered_views
 from property_view import PropertyView
 from xy_plot_view import XYPlotView
-from table_view import TableView
\ No newline at end of file
+
+try:
+    from table_view import TableView
+except:
+    pass
\ No newline at end of file
